unit FrmRandomizer;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.CheckLst, System.IniFiles,

  knobs2013, KnobsUtils, KnobsConversions, FrmStore, Vcl.ComCtrls;

type

  TOnRandomizerLog = procedure( const aSender: TObject; const aMsg: string) of object;

  TFormRandomizer = class(TForm)
    PanelControls: TPanel;
    BitBtnClose: TBitBtn;
    PageControlMain: TPageControl;
    TabSheetSettings: TTabSheet;
    PanelTypes: TPanel;
    PanelControlTypes: TPanel;
    CheckListBoxAllowedControlTypes: TCheckListBox;
    PanelControlTypesControl: TPanel;
    Label1: TLabel;
    BitBtnSelectAllControls: TBitBtn;
    BitBtnDeselectAllControls: TBitBtn;
    PanelModuleTypes: TPanel;
    BitBtnToggleAllControls: TBitBtn;
    PanelModuleStuff: TPanel;
    PanelModuleTypesControl: TPanel;
    Label2: TLabel;
    BitBtnSelectAllModules: TBitBtn;
    BitBtnDeselectAllModules: TBitBtn;
    BitBtnToggleAllModules: TBitBtn;
    CheckListBoxAllowedModuleTypes: TCheckListBox;
    PanelModuleCategories: TPanel;
    PanelModuleCategoriesControl: TPanel;
    Label3: TLabel;
    CheckListBoxAllowedModuleCategories: TCheckListBox;
    RndAmount: TKnobsSmallKnob;
    DisplayRandomizationAmount: TKnobsDisplay;
    Label4: TLabel;
    procedure BitBtnCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure CheckListBoxAllowedControlTypesClickCheck(Sender: TObject);
    procedure CheckListBoxAllowedModuleTypesClickCheck(Sender: TObject);
    procedure BitBtnSelectAllControlsClick(Sender: TObject);
    procedure BitBtnDeselectAllControlsClick(Sender: TObject);
    procedure BitBtnSelectAllModulesClick(Sender: TObject);
    procedure BitBtnDeselectAllModulesClick(Sender: TObject);
    procedure BitBtnToggleAllControlsClick(Sender: TObject);
    procedure BitBtnToggleAllModulesClick(Sender: TObject);
    procedure CheckListBoxAllowedModuleCategoriesClickCheck(Sender: TObject);
    procedure RndAmountValueChanged(const aSender: TObject; const aPath, aControlType: string; aValue: Double; IsFinal,
      IsAutomation: Boolean);
  private
    FPatchEditor                : TKnobsWirePanel;
    FControlTypes               : TKnobsConverters;
    FModuleTypes                : TKnobsModuleList;
    FOrigShowAllowRandomization : Boolean;
    FRndKnobPosition            : Integer;
    FRandomAmount               : TSignal;
    FLogger                     : TOnRandomizerLog;
  private
    procedure   SetRndKnobPosition( aValue: Integer);
  private
    procedure   DeselectAllControlTypes;
    procedure   DeselectAllModuleypes;
    procedure   SelectAllControlTypes;
    procedure   SelectAllModuleTypes;
    procedure   ToggleAllControlTypes;
    procedure   ToggleAllModuleTypes;
    procedure   SyncControlTypes;
    procedure   CollectControlTypes;
    procedure   SyncModuleTypes;
    procedure   CollectModuleTypes;
    procedure   CollectModuleCategories;
    procedure   UpdateControlType   ( anIndex: Integer);
    procedure   UpdateModuleType    ( anIndex: Integer);
    procedure   UpdateModuleCategory( anIndex: Integer);
    procedure   UpdateAllControlTypes;
    procedure   UpdateAllModuleTypes;
    procedure   CloseView;
    procedure   Log( const aMsg: string);
    procedure   LogFmt( const aFmt: string; const anArgs: array of const);
    procedure   SaveControlTypes( const anIniFile: TMemIniFile; const aSection: string);
    procedure   SaveModuleTypes ( const anIniFile: TMemIniFile; const aSection: string);
    procedure   LoadControlTypes( const anIniFile: TMemIniFile; const aSection: string);
    procedure   LoadModuleTypes ( const anIniFile: TMemIniFile; const aSection: string);
  public
    procedure   SaveSettings    ( const anIniFile: TMemIniFile; const aSection: string);
    procedure   LoadSettings    ( const anIniFile: TMemIniFile; const aSection: string);
    procedure   View( aPatchEditor: TKnobsWirePanel);
    procedure   PatchChanged;
  public
    property    RndKnobPosition : Integer          read FRndKnobPosition write SetRndKnobPosition;
    property    RandomAmount    : TSignal          read FRandomAmount;
    property    Logger          : TOnRandomizerLog read FLogger          write FLogger;
  end;

var

  FormRandomizer: TFormRandomizer;



implementation



// User area

const

  sdfs_controltypes = '_ControlTypes';
  sdfs_moduletypes  = '_ModuleTypes';


//  private

    procedure   TFormRandomizer.SetRndKnobPosition( aValue: Integer);
    begin
      if aValue <> FRndKnobPosition
      then begin
        FRndKnobPosition       := aValue;
        RndAmount.KnobPosition := aValue;
        FRandomAmount          := RndAmount.AsValue;

        if Assigned( FPatchEditor)
        then FPatchEditor.RandomAmount := RandomAmount;
      end;
    end;


//  private

    procedure   TFormRandomizer.DeselectAllControlTypes;
    var
      i : Integer;
    begin
      for i := 0 to CheckListBoxAllowedControlTypes.Count - 1
      do CheckListBoxAllowedControlTypes.Checked[ i] := False;

      UpdateAllControlTypes;
    end;


    procedure   TFormRandomizer.DeselectAllModuleypes;
    var
      i : Integer;
    begin
      for i := 0 to CheckListBoxAllowedModuleTypes.Count - 1
      do CheckListBoxAllowedModuleTypes.Checked[ i] := False;

      UpdateAllModuleTypes;
    end;


    procedure   TFormRandomizer.SelectAllControlTypes;
    var
      i : Integer;
    begin
      for i := 0 to CheckListBoxAllowedControlTypes.Count - 1
      do CheckListBoxAllowedControlTypes.Checked[ i] := True;

      UpdateAllControlTypes;
    end;


    procedure   TFormRandomizer.SelectAllModuleTypes;
    var
      i : Integer;
    begin
      for i := 0 to CheckListBoxAllowedModuleTypes.Count - 1
      do CheckListBoxAllowedModuleTypes.Checked[ i] := True;

      UpdateAllModuleTypes;
    end;


    procedure   TFormRandomizer.ToggleAllControlTypes;
    var
      i : Integer;
    begin
      for i := 0 to CheckListBoxAllowedControlTypes.Count - 1
      do CheckListBoxAllowedControlTypes.Checked[ i] := not CheckListBoxAllowedControlTypes.Checked[ i];

      UpdateAllControlTypes;
    end;


    procedure   TFormRandomizer.ToggleAllModuleTypes;
    var
      i : Integer;
    begin
      for i := 0 to CheckListBoxAllowedModuleTypes.Count - 1
      do CheckListBoxAllowedModuleTypes.Checked[ i] := not CheckListBoxAllowedModuleTypes.Checked[ i];

      UpdateAllModuleTypes;
    end;


    procedure   TFormRandomizer.SyncControlTypes;
    var
      i : Integer;
      C : TKnobsNullConverter;
    begin
      if Assigned( FControlTypes)
      then begin
        CheckListBoxAllowedControlTypes.Clear;
        CheckListBoxAllowedControlTypes.Sorted := False;

        for i := 0 to FControlTypes.Count - 1
        do begin
          C := FControlTypes.ConverterObject[ i];

          if Assigned( C)
          then begin
            CheckListBoxAllowedControlTypes.AddItem( C.Name, C);
            CheckListBoxAllowedControlTypes.Checked[ CheckListBoxAllowedControlTypes.Count - 1] := C.IsRandomizable;
          end;
        end;

        CheckListBoxAllowedControlTypes.Sorted := True;
      end;
    end;


    procedure   TFormRandomizer.CollectControlTypes;
    begin
      FControlTypes := Converters;
      SyncControlTypes;
    end;


    procedure   TFormRandomizer.SyncModuleTypes;
    var
      i : Integer;
      M : TKnobsCustomModule;
    begin
      if Assigned( FModuleTypes)
      then begin
        CheckListBoxAllowedModuleTypes.Clear;
        CheckListBoxAllowedModuleTypes.Sorted := False;

        for i := 0 to FModuleTypes.Count - 1
        do begin
          M := FModuleTypes[ i];

          if Assigned( M) and ( M.Title <> '') and ( M.ModuleType >= 0) and ( M.HasRandomizableControls)
          then begin
            CheckListBoxAllowedModuleTypes.AddItem( M.Title, M);
            CheckListBoxAllowedModuleTypes.Checked[ CheckListBoxAllowedModuleTypes.Count - 1] := M.AllowRandomization;
          end;
        end;

        CheckListBoxAllowedModuleTypes.Sorted := True;
      end;
    end;


    procedure   TFormRandomizer.CollectModuleTypes;
    begin
      if Assigned( FormStore)
      then begin
        FModuleTypes := FormStore.Modules;
        SyncModuleTypes;
      end;
    end;


    procedure   TFormRandomizer.CollectModuleCategories;
    var
      aList : TStringList;
      i     : Integer;
    begin
      if Assigned( FormStore)
      then begin
        aList := FormStore.CreatePageNames;

        try
          CheckListBoxAllowedModuleCategories.Clear;

          for i := 0 to aList.Count - 1
          do CheckListBoxAllowedModuleCategories.AddItem( aList[ i], nil);
        finally
          aList.DisposeOf;
        end;
      end;
    end;


    procedure   TFormRandomizer.UpdateControlType( anIndex: Integer);
    var
      S : string;
      R : Boolean;
    begin
      if Assigned( FControlTypes) and Assigned( FPatchEditor)
      then begin
        S := CheckListBoxAllowedControlTypes.Items  [ anIndex];
        R := CheckListBoxAllowedControlTypes.Checked[ anIndex];
        FControlTypes.SetRandomizable( S, R);
        FPatchEditor.SyncControlRandomization( S, R);
      end;
    end;


    procedure   TFormRandomizer.UpdateModuleType( anIndex: Integer);
    var
      S : string;
      R : Boolean;
      M : TKnobsCustomModule;
      T : TKnobsModuleType;
    begin
      if Assigned( FModuleTypes) and Assigned( FPatchEditor)
      then begin
        S := CheckListBoxAllowedModuleTypes.Items  [ anIndex];
        R := CheckListBoxAllowedModuleTypes.Checked[ anIndex];
        M := FModuleTypes.ByName[ S];

        if Assigned( M)
        then begin
          M.AllowRandomization := R;
          T := M.ModuleType;
          FPatchEditor.SyncModuleRandomization( T, R);
        end;
      end;
    end;


    procedure   TFormRandomizer.UpdateModuleCategory( anIndex: Integer);
    var
      i            : Integer;
      j            : Integer;
      R            : Boolean;
      CategoryName : string;
      ModuleName   : string;
    begin
      if Assigned( FModuleTypes) and Assigned( FPatchEditor)
      then begin
        CategoryName := CheckListBoxAllowedModuleCategories.Items  [ anIndex];
        R            := CheckListBoxAllowedModuleCategories.Checked[ anIndex];

        for i := 0 to FModuleTypes.Count - 1
        do begin
          if SameText( FModuleTypes.Module[ i].PageName, CategoryName)
          then begin
            ModuleName := FModuleTypes.Module[ i].Title;

            for j := 0 to CheckListBoxAllowedModuleTypes.Count - 1
            do begin
              if SameText( CheckListBoxAllowedModuleTypes.Items[ j], ModuleName)
              then begin
                CheckListBoxAllowedModuleTypes.Checked[ j] := R;
                Break;
              end;
            end;
          end;
        end;

        UpdateAllModuleTypes;
      end;
    end;


    procedure   TFormRandomizer.UpdateAllControlTypes;
    var
      i : Integer;
    begin
      for i := 0 to CheckListBoxAllowedControlTypes.Count - 1
      do UpdateControlType( i);
    end;


    procedure   TFormRandomizer.UpdateAllModuleTypes;
    var
      i : Integer;
    begin
      for i := 0 to CheckListBoxAllowedModuleTypes.Count - 1
      do UpdateModuleType( i);
    end;


    procedure   TFormRandomizer.CloseView;
    begin
      if Assigned( FPatchEditor)
      then FPatchEditor.ShowAllowRandomization := FOrigShowAllowRandomization;
    end;


    procedure   TFormRandomizer.Log( const aMsg: string);
    begin
      if Assigned( FLogger)
      then FLogger( Self, Format( 'Randomizer: %s', [ aMsg], AppLocale));
    end;


    procedure   TFormRandomizer.LogFmt( const aFmt: string; const anArgs: array of const);
    begin
      Log( Format( aFmt, anArgs, AppLocale));
    end;


    procedure   TFormRandomizer.SaveControlTypes( const anIniFile: TMemIniFile; const aSection: string);
    var
      i : Integer;
    begin
      with anIniFile
      do begin
        EraseSection( aSection);

        if Assigned( FControlTypes)
        then begin
          for i := 0 to FControlTypes.Count - 1
          do WriteBool( aSection, FControlTypes.ConverterObject[ i].ClassName, FControlTypes.ConverterObject[ i].IsRandomizable);
        end;
      end;
    end;


    procedure   TFormRandomizer.SaveModuleTypes( const anIniFile: TMemIniFile; const aSection: string);
    var
      i : Integer;
    begin
      with anIniFile
      do begin
        EraseSection( aSection);

        if Assigned( FModuleTypes)
        then begin
          for i := 0 to FModuleTypes.Count - 1
          do WriteBool( aSection, FModuleTypes[ i].Name, FModuleTypes[ i].AllowRandomization);
        end;
      end;
    end;


    procedure   TFormRandomizer.LoadControlTypes( const anIniFile: TMemIniFile; const aSection: string);
    var
      aList : TStringList;
      i     : Integer;
    begin
      with anIniFile
      do begin
        if Assigned( FControlTypes)
        then begin
          aList := TStringList.Create;

          try
            ReadSection( aSection, aList);

            for i := 0 to aList.Count - 1
            do begin
              FControlTypes.SetRandomizableByClass(
                aList[ i],
                ReadBool(
                  aSection,
                  aList[ i],
                  FControlTypes.GetRandomizableByClass( aList[ i])
                )
              );
            end;
          finally
            aList.DisposeOf;
          end;

         SyncControlTypes;
        end;
      end;
    end;


    procedure   TFormRandomizer.LoadModuleTypes( const anIniFile: TMemIniFile; const aSection: string);
    var
      aList : TStringList;
      i     : Integer;
      M     : TKnobsCustomModule;
    begin
      with anIniFile
      do begin
        if Assigned( FModuleTypes)
        then begin
          aList := TStringList.Create;

          try
            ReadSection( aSection, aList);

            for i := 0 to aList.Count - 1
            do begin
              M := FModuleTypes.ByName[ aList[ i]];

              if Assigned( M)
              then begin
                M.AllowRandomization :=
                  ReadBool(
                    aSection,
                    aList[ i],
                    M.AllowRandomization
                  )
              end;
            end;
          finally
            aList.DisposeOf;
          end;

          SyncModuleTypes;
        end;
      end;
    end;


//  public


    procedure   TFormRandomizer.SaveSettings( const anIniFile: TMemIniFile; const aSection: string);
    begin
      if Assigned( anIniFile)
      then begin
        LogFmt( 'saving settings to: ''%s''', [ anIniFile.FileName]);

        with anIniFile
        do begin
          EraseSection( aSection);
          WriteInteger( aSection, 'left'           , Left           );
          WriteInteger( aSection, 'top'            , Top            );
          WriteInteger( aSection, 'width'          , Width          );
          WriteInteger( aSection, 'height'         , Height         );
          WriteInteger( aSection, 'rndknobposition', RndKnobPosition);
        end;

        SaveControlTypes( anIniFile, aSection + sdfs_controltypes);
        SaveModuleTypes ( anIniFile, aSection + sdfs_moduletypes );
      end;
    end;


    procedure   TFormRandomizer.LoadSettings( const anIniFile: TMemIniFile; const aSection: string);
    var
      L, T, W, H : Integer;
    begin
      if Assigned( anIniFile)
      then begin
        LogFmt( 'loading settings from: ''%s''', [ anIniFile.FileName]);

        with anIniFile
        do begin
          L               := ReadInteger( aSection, 'left'           , Left           );
          T               := ReadInteger( aSection, 'top'            , Top            );
          W               := ReadInteger( aSection, 'width'          , Width          );
          H               := ReadInteger( aSection, 'height'         , Height         );
          RndKnobPosition := ReadInteger( aSection, 'rndknobposition', RndKnobPosition);

          SetBounds( L, T, W, H);
        end;

        LoadControlTypes( anIniFile, aSection + sdfs_controltypes);
        LoadModuleTypes ( anIniFile, aSection + sdfs_moduletypes );
      end;
    end;


    procedure   TFormRandomizer.View( aPatchEditor: TKnobsWirePanel);
    begin
      FPatchEditor := aPatchEditor;
      PatchChanged;

      if Assigned( FPatchEditor)
      then begin
        FOrigShowAllowRandomization         := FPatchEditor.ShowAllowRandomization;
        FPatchEditor.ShowAllowRandomization := True;
        FPatchEditor.RandomAmount           := RandomAmount;
      end;

      Show;
    end;


    procedure   TFormRandomizer.PatchChanged;
    begin
      // Nothing for now
    end;


// Delphi area

{$R *.dfm}

procedure TFormRandomizer.RndAmountValueChanged(const aSender: TObject; const aPath, aControlType: string;
  aValue: Double; IsFinal, IsAutomation: Boolean);
begin
  RndKnobPosition := ( aSender as TKnobsKnob).KnobPosition;
  FRandomAmount   := aValue;
end;

procedure TFormRandomizer.BitBtnDeselectAllControlsClick(Sender: TObject);
begin
  DeselectAllControlTypes;
end;

procedure TFormRandomizer.BitBtnDeselectAllModulesClick(Sender: TObject);
begin
  DeselectAllModuleypes;
end;

procedure TFormRandomizer.BitBtnSelectAllControlsClick(Sender: TObject);
begin
  SelectAllControlTypes;
end;

procedure TFormRandomizer.BitBtnSelectAllModulesClick(Sender: TObject);
begin
  SelectAllModuleTypes;
end;

procedure TFormRandomizer.BitBtnToggleAllControlsClick(Sender: TObject);
begin
  ToggleAllControlTypes;
end;

procedure TFormRandomizer.BitBtnToggleAllModulesClick(Sender: TObject);
begin
  ToggleAllModuleTypes;
end;

procedure TFormRandomizer.CheckListBoxAllowedControlTypesClickCheck(Sender: TObject);
begin
  UpdateControlType( CheckListBoxAllowedControlTypes.ItemIndex);
end;

procedure TFormRandomizer.CheckListBoxAllowedModuleCategoriesClickCheck(Sender: TObject);
begin
  UpdateModuleCategory( CheckListBoxAllowedModuleCategories.ItemIndex);
end;

procedure TFormRandomizer.CheckListBoxAllowedModuleTypesClickCheck(Sender: TObject);
begin
  UpdateModuleType( CheckListBoxAllowedModuleTypes.ItemIndex);
end;

procedure TFormRandomizer.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  CloseView;
end;

procedure TFormRandomizer.FormCreate(Sender: TObject);
begin
  CollectControlTypes;
  CollectModuleTypes;
  CollectModuleCategories;
end;

procedure TFormRandomizer.BitBtnCloseClick(Sender: TObject);
begin
  Close;
end;

end.

